@@ -64,6 +64,26 @@ static const gchar SETTINGS_SCHEMA[] = "org.gnome.Logs";
 static const gchar CLOCK_FORMAT[] = "clock-format";
 static const gchar SORT_ORDER[] = "sort-order";
 
+GtkWidget *
+gl_event_view_list_get_category_list (GlEventViewList *view)
+{
+    GlEventViewListPrivate *priv;
+
+    priv = gl_event_view_list_get_instance_private (view);
+
+    return priv->categories;
+}
+
+const gchar *
+gl_event_view_list_get_boot_match (GlEventViewList *view)
+{
+    GlEventViewListPrivate *priv;
+
+    priv = gl_event_view_list_get_instance_private (view);
+
+    return priv->boot_match;
+}
+
 gchar *
 gl_event_view_list_get_output_logs (GlEventViewList *view)
 {
